""" 
Copyright (c) 2021, Tong Wang, Professorship for Audio Information Processing, Technical University of Munich, Germany. 

This work is licensed under a Creative Commons Attribution 4.0 International (CC BY 4.0) License, see https://creativecommons.org/licenses/by/4.0/ for more information. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import openvr
import argparse
import numpy as np

# from rotations_rtSOFE import euler2rotm
# from scipy.spatial.transform import Rotation as Rot

from pythonosc import udp_client
from pythonosc import osc_bundle_builder
from pythonosc import osc_message_builder

np.set_printoptions(precision = 2)

# setting constants
DURATION =  300 # seconds
REFRESH_RATE = 90 # Hz

# function def moved to file.

# same procedure as in older scripts to create client instances
parser = argparse.ArgumentParser()
parser.add_argument("--ip_1", default='127.0.0.1')
parser.add_argument("--port_1", type=int, default=40000)
parser.add_argument("--ip_2", default='127.0.0.1')
parser.add_argument("--port_2", type=int, default=40001)
args, unknown = parser.parse_known_args()

rt_sofe_client_1 = udp_client.SimpleUDPClient(args.ip_1, args.port_1)
rt_sofe_client_2 = udp_client.SimpleUDPClient(args.ip_2, args.port_2)

command1 = "/receiver-transform" # raise warning if message doesn't start with '/'. Address has to start with '/' according to OSC spec.
command2 = "/receiver-pos"
openvr.init(openvr.VRApplication_Scene)
poses = []  # will be populated with proper type after first call

for i in range(DURATION * REFRESH_RATE):
    poses, _ = openvr.VRCompositor().waitGetPoses(poses, None)
    hmd_pose = poses[openvr.k_unTrackedDeviceIndex_Hmd]

    m = hmd_pose.mDeviceToAbsoluteTracking
    
    # build osc bundle
    osc_bundle = osc_bundle_builder.OscBundleBuilder(osc_bundle_builder.IMMEDIATELY)
    osc_msg_pose = osc_message_builder.OscMessageBuilder(address=command1)
    osc_msg_position = osc_message_builder.OscMessageBuilder(address=command2)

    r_mat_openvr = np.array([[m[0][0], m[0][1], -m[0][2]], [m[1][0], m[1][1], -m[1][2]], [-m[2][0], -m[2][1], m[2][2]]]) # 0. invert some symbols
    r_mat_openvr[:, [1, 2]] = r_mat_openvr[:, [2, 1]] # 1. switch 2nd and 3rd column
    r_mat_openvr[[1, 2], :] = r_mat_openvr[[2, 1], :]# 2. switch 2nd and 3rd row 
    r_mat_openvr = r_mat_openvr.T # 3. transpose

    R = r_mat_openvr
    
    # assign values to args, column major order. rotation matrix
    args = [0] * 9
    args[0] = R[0][0]
    args[1] = R[1][0]
    args[2] = R[2][0]
    args[3] = R[0][1]
    args[4] = R[1][1]
    args[5] = R[2][1]
    args[6] = R[0][2]
    args[7] = R[1][2]
    args[8] = R[2][2]

    # positions with offsets. positions
    args_pos = [0] * 3
    args_pos[0] = 1 + float(m[0][3]) # x
    args_pos[1] = 2 - float(m[2][3]) # y
    args_pos[2] = 0 + float(m[1][3]) # z

    for arg in args:
        osc_msg_pose.add_arg(arg, 'f')  # float number
    osc_bundle.add_content(osc_msg_pose.build()) # build message, add to bundle
    for arg in args_pos:
        osc_msg_position.add_arg(arg, 'f') # float number
    osc_bundle.add_content(osc_msg_position.build())

    b = osc_bundle.build()
    rt_sofe_client_1.send(b)
    rt_sofe_client_2.send(b)

    # debug output
    """
    if i % REFRESH_RATE == 0:
        print("openVR transformation matrix")
        print(m)
        print("transformation matrix sent to rtSOFE")
        print(R)
        print('rtSOFE location (x, y, z)')
        print(args_pos)
    """   
openvr.shutdown()

