%{
Copyright (c) 2021, Tong Wang, Professorship for Audio Information Processing, Technical University of Munich, Germany. 

This work is licensed under a Creative Commons Attribution 4.0 International (CC BY 4.0) License, see https://creativecommons.org/licenses/by/4.0/ for more information. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%}

% Interactive mode: Model-in-the-loop pipeline: start rtSOFE in Matlab, send
% positions to rtSOFE, receive IR in Matlab, plot IR;
clearvars client
%add paths: path for release with MATLAB utils, 2nd path for test
%environment.
addpath('..\..\..\..\rtSOFE_Matlab_Utils')
addpath('Y:\resources\rtSOFE\rtSOFE_mcIR_MATLAB_receiver')
% dependency: mk_osc_msg.m

%% constants
rtsofe_port = 6500; % rtSOFE process #1 early reflections
osc_port = 40000; % rtSOFE process #1 early reflections
rtsofe_IP = '127.0.0.1';
launch_sofe = true;
demo_plot = true;
rtsofe_path = '..\..\..\Executables\rt_sofe.exe';
rtsofe_config = 'rtsofe-config03.ini';
time_before_tcp_client_creation = 15; % seconds
%% start rtSOFE
win_cmd = ['start /high ', rtsofe_path, ' --config ', rtsofe_config];
if launch_sofe
    system(win_cmd);
    % launches rtSOFE in interactive mode.
    pause(time_before_tcp_client_creation);
    % wait for rtSOFE to init. The first launch can be slow as FFTW3 try to
    % decided the fastest FFT algorithm and generate a "Wisdom" file on the
    % current PC. If a fftw_wisdom file can be found, rtSOFE can skip the 
    % FFTW alogrithm benchmark and initialize faster.
end
%% start tcp client; launch rtSOFE in interactive mode first
client = tcpclient("localhost",rtsofe_port);
pause(1);
mcIRs = {}; % cell array for mcIRs
oscUDP = udpport();

%% Option 1: create osc msg, send with udpport(), send osc packets containing 1 single parameter (source position) update;

traj = ([2.1 3.1 2; 3 3 2; 1 3.2 2]);
% read and discard the first mcIR which uses the positions in the config file.
[~] = read_mcIR_from_stream(client); 
figure; title('multi-channel IR'); xlabel('sample'); ylabel('amplitude')
for i = 1: size(traj,1)
    % mk_osc_msg(path,types,arg1,arg2,...), see mk_osc_msg.m for usage.
    data = mk_osc_msg('/source-pos', 'fff', traj(i,1), traj(i,2), traj(i,3));
    write(oscUDP, data, rtsofe_IP, osc_port); % send 
    pause(0.01);
    mcIRs{end+1} = read_mcIR_from_stream(client); 
    % you can also write a function to wait longer or resend the position
    % if there is no data received from rtSOFE.
    
    % demo only, do a plot for each received mcIR
    if demo_plot
        plot(mcIRs{end}); ylim([-1,1]); xlim([0, 4000])
        disp('Press any key to send the next position');
        pause; % send the next after manual resume in MATLAB console. can remove.
    end
end

%% Option 2: create osc msgs, make an osc bundle, send an osc bundle
% Update two parameters (for demo: source and receiver position), both of
% which will be used to compute the next IR.
traj_src = ([2.1 3.1 2; 3 3 2; 1 3.2 2]);
traj_rec = ([1.1 3.1 2; 2 3 2; 1 2.2 2]);

for i = 1: size(traj_rec,1)
    % mk_osc_msg(path,types,arg1,arg2,...), see mk_osc_msg.m for usage.
    msg1 = mk_osc_msg('/source-pos', 'fff', traj_src(i,1), traj_src(i,2), traj_src(i,3));
    msg2 = mk_osc_msg('/receiver-pos', 'fff', traj_rec(i,1), traj_rec(i,2), traj_rec(i,3));
    bundle = mk_osc_bundle(msg1);
    bundle = mk_osc_bundle(msg2, bundle);
    
    write(oscUDP, bundle, rtsofe_IP, osc_port); % send 
    pause(0.01);
    mcIRs{end+1} = read_mcIR_from_stream(client); 
    % you can also write a function to wait longer or resend the position
    % if there is no data received from rtSOFE.
    
    % demo only, do a plot for each received mcIR
    if demo_plot
        plot(mcIRs{end}); ylim([-1,1]); xlim([0, 4000])
        pause; % send the next after manual resume in MATLAB console. can remove.
    end
end

%% Exit via OSC
disp('Press any key to exit rtsofe');
msg_exit = mk_osc_msg('/exit');
write(oscUDP, msg_exit, rtsofe_IP, osc_port);