%{
Copyright (c) 2021, Tong Wang, Professorship for Audio Information Processing, Technical University of Munich, Germany. 

This work is licensed under a Creative Commons Attribution 4.0 International (CC BY 4.0) License, see https://creativecommons.org/licenses/by/4.0/ for more information. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function osc_bundle = mk_osc_bundle(osc_obj, osc_bundle_original) 
%}

% This function insert an osc message or bundle to an existing osc bundle
% according to the osc spec 1.0. In case the existing osc bundle is not
% provided, this function creates a new osc bundle.
% ref: http://opensoundcontrol.org/spec-1_0.html#osc-bundles
% 18.08.2021 Tong Wang. Function for type cast is created by MARK MARIJNISSEN in oscsend.m; 
% status:
% 1. Tested, works. except timetag: not processed at the receiver end.
% 2. which "time" to use for timetag??

    obj_size = oscint(length(osc_obj));
    if (exist('osc_bundle_original', 'var'))
        osc_bundle = [osc_bundle_original obj_size osc_obj];
    else
        osc_string = ['#bundle', 0];
        D = datetime;
        timetag = osc_timetag(convertTo(D, 'ntp'));
        osc_bundle = [osc_string timetag obj_size osc_obj];
    end
end

%Conversion to int
function int = oscint(int)
    [~, ~, endian] = computer;
    littleEndian = endian == 'L';
   if littleEndian
        int = typecast(swapbytes(int32(int)),'uint8');
   else
        int = typecast(int32(int),'uint8');
   end
end

function int = osc_timetag(NTP_timetag)
    [~, ~, endian] = computer;
    littleEndian = endian == 'L';
   if littleEndian
        int = typecast(swapbytes(int64(NTP_timetag)),'uint8');
   else
        int = typecast(int64(int),'uint8');
   end
end