%{
Copyright (c) 2021, Tong Wang, Professorship for Audio Information Processing, Technical University of Munich, Germany. 

This work is licensed under a Creative Commons Attribution 4.0 International (CC BY 4.0) License, see https://creativecommons.org/licenses/by/4.0/ for more information. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function osc_bundle = mk_osc_bundle(osc_obj, osc_bundle_original) 
%}
function mcIR = read_mcIR_from_stream(client_name)
    % read multichannel-IR from bitstream sent by rtSOFE in TCPMatrix
    % format. 

    % Read different parts of data; first part is unknown data
    % return one multi channel IR
    %
    % get rid of irrelevant data of 13 bytes. serializer header.
    % referece: tcp_input.h
    header_num_bytes = [8 5]; % TCP matrix are stored in the later part of the data stream.
    % the size can be infered by trasmitting one TCP matrix and compared it
    % with the IR generated using the offline mode.
    
    %===========  what is know about the 13 bytes?
    %first 8 bytes: headers. size of data stream saved in HEX
    %next 5 bytes:  always 0 1 0 0 0 in uint 8, no idea what they are.
    %probably added by the boost binary archive function. no use.
    %according to pad_matrix.h, probably version of TCP Matrix(not sure)
    %===========
    
    % if this number is not a constant, find another way to decide.
    datasize = hex2dec(read(client_name, header_num_bytes(1),"char")); % ASCII: 32: white space. 
    unknown_data = read(client_name, header_num_bytes(2));
    
    % refer to Hendrik's paper for definition of TCP matrix IR format.
    % Properties of the object are serialized sequentially
    % read offset; refer to Hendrik's paper for definition of TCP matrix IR format
    num_symbols = 1;
    data_type = "uint64";
    IR_offset_sample = read(client_name, num_symbols, data_type);
    %disp(IR_offset_sample)

    % read length per channel
    num_symbols = 1;
    data_type = "uint64";
    len_per_chn = read(client_name, num_symbols, data_type);
    %disp(len_per_chn)

    % read number of channels
    num_symbols = 1;
    data_type = "uint64";
    num_chns = read(client_name, num_symbols, data_type);
    %disp(num_chns)

    % read soundsourceID % not always set, default is 0
    num_symbols = 1;
    data_type = "uint64";
    sound_source_ID = read(client_name, num_symbols, data_type);
    %disp(sound_source_ID)

    % read timestamp % not always set, default is 0
    num_symbols = 1;
    data_type = "uint64";
    mcIR_timestamp = read(client_name, num_symbols, data_type);
    %disp(mcIR_timestamp)
    % read samples
    data = read(client_name, num_chns*len_per_chn, "single");

    % assemble multi_chn IR from data, meta data
    mcIR_offset = zeros(IR_offset_sample, num_chns);
    mcIR_data = reshape(data, [], num_chns );
    mcIR = [mcIR_offset; mcIR_data];
end